package com.supervision.manage.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.extension.conditions.query.LambdaQueryChainWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.supervision.exception.BusinessException;
import com.supervision.manage.service.DiseaseManageService;
import com.supervision.model.Disease;
import com.supervision.service.DiseaseService;
import com.supervision.vo.manage.DiseaseVo;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Objects;
@Service
@RequiredArgsConstructor
public class DiseaseManageServiceImpl implements DiseaseManageService {

    private final DiseaseService diseaseService;
    @Override
    public Disease saveDisease(Disease disease) {

        if (StrUtil.hasEmpty(disease.getDiseaseName(),disease.getDiseaseNameAlias(),disease.getCode())){
            throw new BusinessException("diseaseName,diseaseNameAlias,code is not allow empty");
        }
        if (CollUtil.isNotEmpty(this.queryByAnyCondition(disease.getCode(),disease.getDiseaseName(),
                disease.getDiseaseNameAlias()))){
            throw new BusinessException("类目编码: "+disease.getCode()+",或疾病名称: "+disease.getDiseaseName()+
                    ",或疾病分类别名: "+disease.getDiseaseNameAlias()+" 已存在");
        }

        disease.setId(null);
        disease.setStatus(0);
        diseaseService.save(disease);

        return disease;

    }

    @Override
    public boolean updateDisease(Disease disease) {

        if (StrUtil.isEmpty(disease.getId())){
            throw new BusinessException("id is not allow empty");
        }

        return diseaseService.updateById(disease);

    }

    @Override
    public boolean deleteDisease(String id) {

        //todo: 应该删除其他关联信息

        return diseaseService.removeById(id);
    }

    @Override
    public Page<Disease> queryPageList(Integer pageNum, Integer pageSize, DiseaseVo diseaseVo) {


        LambdaQueryChainWrapper<Disease> wrapper = diseaseService.lambdaQuery()
                .like(StrUtil.isNotEmpty(diseaseVo.getDiseaseName()), Disease::getDiseaseName, diseaseVo.getDiseaseName());

        return wrapper.page(new Page<>(pageNum, pageSize));
    }



    private List<Disease> queryByAnyCondition(String code, String diseaseName, String diseaseNameAlias){

        return diseaseService.lambdaQuery().eq(StrUtil.isNotEmpty(code),Disease::getCode,code).or()
                .eq(StrUtil.isNotEmpty(diseaseName), Disease::getDiseaseName,diseaseName).or()
                .eq(StrUtil.isNotEmpty(diseaseNameAlias),Disease::getDiseaseNameAlias,diseaseNameAlias)
                .list();

    }
    private List<Disease> queryByCode(String code){
        if (Objects.isNull(code)){
            throw new BusinessException("code is not allow empty");
        }
        return diseaseService.lambdaQuery().eq(Disease::getCode,code).list();
    }

}
